@@ -15,9 +15,10 @@ from multiprocessing import Pool
 import os
 from pathlib import Path
 import pickle
-from random import choice
+from random import choice, choices
 import re
 import smtplib
+from string import ascii_letters
 from subprocess import run, check_output
 import sys
 import time
@@ -124,6 +125,80 @@ def hasarg(func, arg):
     return arg in signature(func).parameters
 
 
+# def quickmail(subject, message, to_email, from_email=None,
+#               attach_paths=(), verbose=True, password=None):
+#     """Send an email.
+#
+#     Parameters
+#     -----------
+#     subject: str
+#         Subject line of email.
+#     message: str
+#         Body of email.
+#     to_email: str
+#         Recipient's email. This can also be a verizon phone number in the form
+#         3332221111@vtext.com (notice no extra leading 1), in which case this
+#         will send an sms. In theory, you should be able to send mms
+#         (i.e. include image(s)) by using a format like 3332221111@vzwtext.com,
+#         but this doesn't seem to be working as of 2/10/22.
+#     from_email: str
+#         Gmail address being used to send email.
+#     attach_paths: str or listlike
+#         Paths to files to attach. Currently supports text (.txt, .md, etc.;
+#         as of 2/11/22, gmail blocks executable attachments like .py)
+#         and image (.jpg, .png, etc.) files.
+#
+#     Returns
+#     --------
+#     None
+#     """
+#     # Load email username. Error handling takes place in config functions.
+#     from_email = from_email or get_default_user()
+#     if not from_email: return None
+#
+#     # Load email password.
+#     password = password or get_credentials(from_email)
+#     if not password: return None
+#
+#     # Create message and add text if specified.
+#     msg = MIMEMultipart()
+#     msg['Subject'] = subject
+#     msg['From'] = from_email
+#     msg['To'] = to_email
+#     if message: msg.attach(MIMEText(message))
+#
+#     # Load and attach file(s). Paths must be strings, not Paths, for mimetypes.
+#     for path in map(str, tolist(attach_paths)):
+#         ftype = mimetypes.guess_type(path)[0].split('/')[0]
+#         if ftype == 'text':
+#             mime_cls = MIMEText
+#             mode = 'r'
+#             encoder = identity
+#             kwargs = {}
+#         elif ftype == 'image':
+#             mime_cls = MIMEImage
+#             mode = 'rb'
+#             encoder = encoders.encode_base64
+#             kwargs = {'name': os.path.basename(path)}
+#         else:
+#             raise ValueError('Attached file should be a text or image file. '
+#                              f'We parsed your file as type {ftype}.')
+#         with open(path, mode) as f:
+#             attachment = mime_cls(f.read(), **kwargs)
+#         # show up as an attachment rather than an embedded object, but
+#         # sometimes the later might be preferable.
+#         attachment.add_header('Content-Disposition', 'attachment',
+#                               filename=os.path.basename(path))
+#         encoder(attachment)
+#         msg.attach(attachment)
+#
+#     # Access server and send email.
+#     server = smtplib.SMTP(host='smtp.gmail.com', port=587)
+#     server.starttls()
+#     server.login(user=from_email, password=password)
+#     server.sendmail(from_email, to_email, msg.as_string())
+#     if verbose: print(f'Email sent to {to_email}.')
+
 def quickmail(subject, message, to_email, from_email=None,
               attach_paths=(), verbose=True, password=None):
     """Send an email.
@@ -192,7 +267,14 @@ def quickmail(subject, message, to_email, from_email=None,
         msg.attach(attachment)
 
     # Access server and send email.
-    server = smtplib.SMTP(host='smtp.gmail.com', port=587)
+    hosts = {'gmail.com': 'smtp.gmail.com',
+             'outlook.com': 'smtp-mail.outlook.com'}
+    try:
+        host = hosts[from_email.split('@')[-1]]
+    except KeyError:
+        raise ValueError(f'Unrecognized host {from_email.split("@")[-1]}. '
+                         f'We currently support {hosts.keys()}.')
+    server = smtplib.SMTP(host=host, port=587)
     server.starttls()
     server.login(user=from_email, password=password)
     server.sendmail(from_email, to_email, msg.as_string())
@@ -1291,7 +1373,7 @@ def xor_none(*args, n=1):
     return a value so we get None.
     """
     if sum(bool(arg is not None) for arg in args) != n:
-        raise ValueError(f'Exactly {n} or args must be not None.')
+        raise ValueError(f'Exactly {n} of args must be not None.')
 
 
 def max_key(d, fn=identity):
@@ -1416,6 +1498,19 @@ def spacer(char='-', n_chars=79, newlines_before=1, newlines_after=1):
     return '\n'*newlines_before + char * n_chars + '\n'*newlines_after
 
 
+def hr(char='-', n_chars=79, newlines_before=1, newlines_after=1):
+    """Horizontal rule: print a line, usually to visually separate printed
+    outputs in a for loop. Same as spacer() but this includes the print call
+    instead of just returning a string. Basically, I want to keep spacer to
+    support old code that relies on it, but I wanted something a little
+    quicker/simpler than calling print(spacer()).
+    """
+    print(
+        spacer(char, n_chars,
+               newlines_before=newlines_before, newlines_after=newlines_after)
+    )
+
+
 def func_name(func):
     """Usually just returns the name of a function. The difference is this is
     compatible with functools.partial, which otherwise makes __name__
@@ -1702,5 +1797,25 @@ def set_summary(x1, x2, info=('first_only', 'second_only')):
     return select(res, keep=list(info))
 
 
+def random_str(length, lower=False, valid=tuple(ascii_letters + '0123456789')):
+    """Generate random string of alphanumeric characters.
+
+    Parameters
+    ----------
+    length: int
+        Number of characters in output string.
+    lower: bool
+        If True, the output will be lowercased.
+    valid: Iterable
+        List-like container of valid characters (strings) to sample from.
+
+    Returns
+    -------
+    str: `length` characters long.
+    """
+    text = ''.join(choices(valid, k=length))
+    return text.lower() if lower else text
+
+
 SENTINEL = object()
 
